require 'httpclient'
require 'json'

module Metrorail
    @@api_key = nil
    def self.set_api_key(key)
        @@api_key = key
        puts "Updating station cache data."
        Metrorail::Station.update_cache
        puts "Updating line cache data."
        Metrorail::Line.update_cache
        puts "Done."
    end

    # A class representing a location (i.e. latitude/longitude)
    class Location
        attr_reader :lat, :lon
        def initialize(lat, lon)
            @lat = lat
            @lon = lon
        end

        def to_s
            return "<Location: #{@lat}, #{@lon}>"
        end
    end

    # A class representing a station in the Metrorail system.
    class Station
        attr_reader :id, :name, :lines_served, :location, :is_transfer
        @@stations = []

        def initialize(id, name, lines_served, location)
            @id = [id]
            @name = name
            @lines_served = lines_served
            @location = location
            @is_transfer = false
        end

        def add_id(id)
            @id << id
        end

        def add_line_served(line)
            if not @lines_served.include? line
                @lines_served << line
            end
        end

        def to_s
            return "<Station: #{@name} (#{@id.join(",")}) serving #{@lines_served.join(",")} at #{@location.to_s}>"
        end

        def self.all(force=false)
            if not force and @@stations.size > 0
                return @@stations
            end

            stations_request = Metrorail::make_request("Rail", "jStations")["Stations"]
            stations = []

            stations_request.each do |station|
                same_station = stations.select { |s| s.name == station["Name"] }
                if same_station.size > 0
                    old_station = same_station[0]
                    old_station.add_id station["Code"]
                    ["LineCode1", "LineCode2", "LineCode3", "LineCode4"].each do |linecode|
                        if station[linecode] != nil
                            old_station.add_line_served(station[linecode])
                        end
                    end
                else
                    new_station = Station.new(station["Code"], station["Name"], [], Location.new(station["Lat"], station["Lon"]))
                    ["LineCode1", "LineCode2", "LineCode3", "LineCode4"].each do |linecode|
                        if station[linecode] != nil
                            new_station.add_line_served(station[linecode])
                        end
                    end
                    stations << new_station
                end
            end

            return stations
        end

        def self.find_by_id(id)
            self.update_cache_if_needed
            matching = self.all.select { |station| station.id.include? id }
            if matching.size > 0
                matching[0]
            else
                nil
            end
        end

        def self.update_cache
            @@stations = self.all(true)
        end

        def self.update_cache_if_needed
            if @@stations.size == 0
                self.update_cache
            end
        end
    end

    # A class representing a line in the Metrorail system.
    class Line
        attr_reader :id, :name, :stations
        @@lines = []

        def initialize(id, name, stations=[])
            @id = id
            @name = name
            @stations = []
        end

        def add_station(station)
            @stations << station
        end

        def self.all(force=false)
            if @@lines.size > 0
                return @@lines
            else
                lines_request = Metrorail::make_request("Rail", "jLines")["Lines"]
                lines = []
                lines_request.each do |line|
                    new_line = Line.new(line["LineCode"], line["DisplayName"])
                    stations = Metrorail::make_request("Rail", "jPath", {fromStationCode: line["StartStationCode"], toStationCode: line["EndStationCode"]})["Path"]
                    stations.each do |station|
                        new_line.add_station(Metrorail::Station.find_by_id(station["StationCode"]))
                    end
                    lines << new_line
                end
                return lines
            end
        end

        def self.find_by_id(id)
            self.update_cache_if_needed
            matching = self.all.select { |line| line.id == id }
            if matching.size > 0
                matching[0]
            else
                nil
            end
        end

        def self.update_cache
            @@lines = self.all(true)
        end

        def self.update_cache_if_needed
            if @@lines.size == 0
                self.update_cache
            end
        end
    end

    private
    def self.make_request(mod, submod, args={})
        client = HTTPClient.new
        args["api_key"] = @@api_key
        JSON::parse(client.get("http://api.wmata.com/#{mod}.svc/json/#{submod}", args).body)
    end
end
